/***********************************************************************************************************************
 *
 * blueBill Mobile - Android - open source birding
 * Copyright (C) 2009-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://bluebill.tidalwave.it/mobile
 * SCM: https://java.net/hg/bluebill-mobile~android-src
 *
 **********************************************************************************************************************/
package it.tidalwave.bluebill.factsheet.bbc.io;

import it.tidalwave.bluebill.factsheet.Biblio;
import it.tidalwave.bluebill.factsheet.FactSheet;
import it.tidalwave.bluebill.factsheet.bbc.BbcFactSheetProvider;
import it.tidalwave.mobile.media.Media;
import it.tidalwave.mobile.util.Downloadable;
import it.tidalwave.semantic.io.GraphDeserializer;
import it.tidalwave.semantic.io.MimeTypes;
import it.tidalwave.semantic.io.impl.DefaultGraphDeserializer;
import it.tidalwave.util.As;
import it.tidalwave.util.Id;
import it.tidalwave.util.Key;
import java.io.BufferedInputStream;
import java.io.FileInputStream;
import java.io.InputStream;
import java.util.List;
import javax.annotation.Nonnull;
import org.openrdf.model.Graph;
import org.openrdf.model.Statement;
import org.openrdf.model.Value;
import static it.tidalwave.mobile.util.Downloadable.Downloadable;
import static it.tidalwave.semantic.vocabulary.DublinCoreVocabulary.*;
import static it.tidalwave.semantic.vocabulary.FoafVocabulary.*;
import static it.tidalwave.semantic.vocabulary.RdfVocabulary.*;
import static it.tidalwave.semantic.vocabulary.RdfsVocabulary.*;
import static it.tidalwave.bluebill.factsheet.bbc.vocabulary.BbcVocabulary.*;

/***********************************************************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 **********************************************************************************************************************/
public class FactSheetUnmarshaller extends StatementUnmarshallerSupport
  {
    @Nonnull
    public As unmarshal (final @Nonnull List<Statement> statements, final @Nonnull Context context)
      {
        final Id id = findId(statements, context);
        FactSheet factSheet = new FactSheet(id).with(DC_PUBLISHER, BbcFactSheetProvider.BBC)
                                               .with(DC_RIGHTS, BBC_SYNDICATION_GUIDELINES);

        for (final Statement statement : statements)
          {
            final String predicate = statement.getPredicate().stringValue();
            final Value object = statement.getObject();

            if (predicate.equals(TYPE_WO_ADAPTATION.stringValue()))
              {
                final Key<Object> key = new Key<Object>(predicate);
                factSheet = factSheet.with(key, context.find(object));
              }
            else if (predicate.equals(TYPE_WO_LIVES_IN.stringValue()))
              {
                final Key<Object> key = new Key<Object>(predicate);
                factSheet = factSheet.with(key, context.find(object));
              }
            else if (predicate.startsWith(NS_WO)
                  || predicate.startsWith(NS_FOAF)
                  || predicate.startsWith(NS_DC_TERMS))
              {
                final Key<Object> key = new Key<Object>(predicate);
                factSheet = factSheet.with(key, deserialize(object));
              }
            else if (!predicate.equals(TYPE_RDF_TYPE.stringValue())
                     && !predicate.equals(RDFS_LABEL.stringValue()))
              {
                System.err.println("SPECIES Unused statement: " + statement);
              }
          }

        for (final Statement statement : context.findStatements(ID_FOAF_DEPICTS, id))
          {
            factSheet = factSheet.with(IMAGE, new Media().with(Media.ID, new Id(context.find(statement.getSubject()))));
          }

        for (final Statement statement : context.findStatements(ID_FOAF_PRIMARY_TOPIC, id))
          {
            factSheet = factSheet.with(BIBLIO, new Biblio(new Id(context.find(statement.getSubject()))));
          }

        for (final Statement statement : context.findStatements(ID_PO_SUBJECT, id))
          {
            Media movie = (Media)context.find(statement.getSubject());

            try
              {
                final Id movieId = new Id(movie.get(Media.ID).stringValue().replace("#programme", ".rdf"));
                final Media m = new Media().with(Media.ID, movieId);
                final Downloadable downloadable = m.as(Downloadable);
                downloadable.download();
                downloadable.waitUntilDownloadingCompleted();
                final GraphDeserializer deserializer = new DefaultGraphDeserializer();
                deserializer.setContextUri(BBC_CONTEXT);
                final InputStream is = new BufferedInputStream(new FileInputStream(downloadable.getFile()), 8192);
                final Graph graph = deserializer.read(is, MimeTypes.MIME_RDF_XML);
                is.close();

                for (final Statement statement3 : graph)
                  {
                    if (statement3.getPredicate().stringValue().equals("http://purl.org/ontology/po/medium_synopsis"))
                      {
                        movie = movie.with(DC_ABSTRACT, statement3.getObject().stringValue());
                      }
                    if (statement3.getPredicate().stringValue().equals("http://purl.org/ontology/po/long_synopsis"))
                      {
                        movie = movie.with(DC_DESCRIPTION, statement3.getObject().stringValue());
                      }
                  }
              }
            catch (Exception e)
              {
                e.printStackTrace(); // FIXME
              }

            factSheet = factSheet.with(MOVIE, movie);
          }

        return factSheet;
      }
  }
